
//Kamera_Blitz Steuereinheit
//Für Arduino MICRO
//Erstellt von Alex Merz


//////////////////////////////////////////Include
#include <RotaryEncoderWithButton.h>
#include <LiquidCrystal.h>


//////////////////////////////////////////Definitionen
///////////Pinbelegung
//LCD
#define LCD_RS          12
#define LCD_Enable      11
#define LCD_D4          5
#define LCD_D5          4
#define LCD_D6          3
#define LCD_D7          2
 
//Encoder
#define ENC_1           1   //CLK
#define ENC_2           0   //DT
#define ENC_btn         6   //Taster Encoder
 
//Kamera/Blitz-wecken
#define wakeup          A1    //Um Kamera (fokuspin) und evtl Blitz aus Schlafmodus zu holen

//Kamera
#define Kamera          A8 

//Blitz
#define Blitz1          A9
#define Blitz2          A10
#define Blitz3          A0  //Wenn es der Alte Canon-Blitz ist, dann benögtigt er 40ms um schalten zu können. 
                            //Ist in Programm zur Sicherheit drin. Verändert Bild jedoch nicht. 
                            //Es ist aber Wichtig, dass der Canon-Blitz auf Platz 3 ist.

//Taster
#define taster_ausloesen 13

//Lichtschranke Tropfen
#define L1               7

//Sensor
#define Sensor          A5

//Ventile
#define Ventil1         A2 //Hier wird auch der Lautsprecher angeschlossen
#define Ventil2         A3
#define Ventil3         A4


///////////Konstanten
#define stepsize 1 //1ms Delayschritte
#define ventildelay 10 //10ms Verzögerung, damit Ventil schalten kann. Mit dieser Zeit wird auch die Tropfengrösse bestimmt.
unsigned long LCDinterval = 150; //LCD alle 150ms aktualisieren. Sonst Flimmerts
#define waehldelay 5000 //5000ms Zeit um Programm zu wählen
#define wakeuptime 40 //40ms damit Kamera genug Zeit hat um zu schalten
#define kameraVerzoegerung 1000 //1s damit Kamera genug Zeit hat um zu schalten. Unter der Annahme Spiegelvorauslösung ist drin (Mup).
#define canonBlitzVerzoegerung 40 //40ms damit Canon-Blitz genug Zeit hat um zu schalten


///////////Variabeln
unsigned long LCDpreviousMillis = 0;

bool initial = true;

int anzahlVentile = 0;

int wechselpos = 0;
long encoderveraenderung = 0;
long oldPosition  = 0;
long newPosition;

int testNr = 0;

int sensorempfindlichkeit = 5;
long int sensorWert;
int maxSensorWert;
int minSensorWert;
int aktualSensorWert;
int anzahlMessungen = 200;
int wertRegister[200]; //Gleichviele Speicherstellen wie "anzahlMessungen"

long kameratime = 0;

bool prgAendern = true;
int prg = 0;
unsigned long waehlzeit;

int ventil1Test = 0;
int ventil2Test = 0;
int ventil3Test = 0;

long timeTropfen2 = 0; 
long timeTropfen3 = 0; 

long blitztime1 = 0;
long blitztime2 = 0;
long blitztime3 = 0;


///////////Klassenaufruf
LiquidCrystal lcd(LCD_RS, LCD_Enable, LCD_D4, LCD_D5, LCD_D6, LCD_D7);
RotaryEncoderWithButton rotary(ENC_1,ENC_2,ENC_btn);


///////////Funktiondeklaration
void tropfenBilderAufnehmen(void);
void LCDaktuell1(void);
void LCDaktuell2(void); 
void LCDaktuell3(void); 
void momentaufnahme(void);
void systemTesten(void);
int encoder(int);
void initialisiere(void);
void tropfenexplosion(void);


///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
///////////Initialisierung
void setup()
{
///////////Serial Monitor starten
  Serial.begin(9600);

///////////Kamera und Blitz
  pinMode(wakeup, OUTPUT);

///////////Kamera
  pinMode(Kamera, OUTPUT);

///////////Blitz
  pinMode(Blitz1, OUTPUT);
  pinMode(Blitz2, OUTPUT);
  pinMode(Blitz3, OUTPUT);

///////////Ventile
  pinMode(Ventil1, OUTPUT);
  pinMode(Ventil2, OUTPUT);
  pinMode(Ventil3, OUTPUT);
  
///////////Rotary Input
  rotary.begin();
  
///////////Taster
  pinMode(taster_ausloesen,INPUT_PULLUP);
  
///////////LCD 
  lcd.begin(16, 4);

}
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
///////////Main
void loop() 
{
  if(initial == true)
  {
    initialisiere();
    initial = false;
  }
  
  //Programm System testen  
  if(prg == 0)
  {
    systemTesten(); //Funktion SystemTesten aufrufen
  }

  //Programm Tropfen Bilder
  if(prg == 1) 
  {
    //Abfrage Bild aufnehmen
    if(digitalRead(taster_ausloesen) == 0)
    {
      tropfenBilderAufnehmen(); //Bild aufnehmen
    }

    //LCD Aufruf/aktualisieren
    encoderveraenderung = encoder(encoderveraenderung);
    LCDaktuell1();
  }

  //Programm Momentaufnahme
  if(prg == 2) 
  {
    //Abfrage Bild aufnehmen
    if(digitalRead(taster_ausloesen) == 0)
    {
      momentaufnahme(); //Bild aufnehmen
    }
    
    //LCD Aufruf/aktualisieren
    encoderveraenderung = encoder(encoderveraenderung);
    LCDaktuell2();    
  }

  //Programm Tropfenexplosion
  if(prg == 3) 
  {
    //Abfrage Bild aufnehmen
    if(digitalRead(taster_ausloesen) == 0)
    {
      tropfenexplosion(); //Bild aufnehmen
    }
    
    //LCD Aufruf/aktualisieren
    encoderveraenderung = encoder(encoderveraenderung);
    LCDaktuell3();    
  }
  
}
///////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////// 
///////////Funktionen
//Kamera-Blitz-Ventil auslösung
void tropfenBilderAufnehmen()
{
  Serial.println("\n");
  Serial.println("Bild wird aufgenommen");
  
  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("Bild wird");

  lcd.setCursor(0, 1);
  lcd.print("aufgenommen");

  lcd.setCursor(0, 3);
  lcd.print("Prozess laeuft..");


  switch(anzahlVentile)
  {
    ////////Ventil 1
    case(0):
    //Kamera
    digitalWrite(wakeup, HIGH);
    delay(wakeuptime);
    digitalWrite(Kamera, HIGH);
    delay(kameraVerzoegerung);
    //Serial.println("Kamera Ausgeloest"); //Später auskommentieren

    //Ventil 1
    digitalWrite(Ventil1, HIGH);
    delay(ventildelay);
    digitalWrite(Ventil1, LOW);
    //Serial.println("Ventil 1 kurz geoeffnet"); //Später auskommentieren

    //Blitz 1
    if(blitztime1 != 0)
    {
      delay(blitztime1);
      digitalWrite(Blitz1, HIGH);
      //Serial.println("Blitz 1 ausgeloest"); //Später auskommentieren 
    }   

    //Tropfen 2
    if(timeTropfen2 != 0)
    {
      delay(timeTropfen2);

      //Ventil 1
      digitalWrite(Ventil1, HIGH);
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      //Serial.println("Ventil 1 kurz geoeffnet"); //Später auskommentieren

      //Blitz 2
      if(blitztime2 != 0)
      {
        delay(blitztime2);
        digitalWrite(Blitz2, HIGH);
        //Serial.println("Blitz 2 ausgeloest"); //Später auskommentieren 
      }
    }

    //Tropfen 3
    if(timeTropfen3 != 0)
    {
      delay(timeTropfen3);

      //Ventil 1
      digitalWrite(Ventil1, HIGH);
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      //Serial.println("Ventil 1 kurz geoeffnet"); //Später auskommentieren

      //Blitz 3
      if(blitztime3 != 0)
      {
        delay(blitztime3);
        digitalWrite(Blitz3, HIGH);
        //Serial.println("Blitz 3 ausgeloest"); //Später auskommentieren 
      }
    }    
    break;

    
    ////////Ventil 1/2
    case(1):
    //Kamera
    digitalWrite(wakeup, HIGH);
    delay(wakeuptime);
    digitalWrite(Kamera, HIGH);
    delay(kameraVerzoegerung);
    //Serial.println("Kamera Ausgeloest"); //Später auskommentieren

    //Ventil 1 und Ventil 2
    digitalWrite(Ventil1, HIGH);
    digitalWrite(Ventil2, HIGH); 
    delay(ventildelay);
    digitalWrite(Ventil1, LOW);
    digitalWrite(Ventil2, LOW);
    //Serial.println("Ventil 1 und Ventil 2 kurz geoeffnet"); //Später auskommentieren

    //Blitz 1
    if(blitztime1 != 0)
    {
      delay(blitztime1);
      digitalWrite(Blitz1, HIGH);
      //Serial.println("Blitz 1 ausgeloest"); //Später auskommentieren 
    }   

    //Tropfen 2
    if(timeTropfen2 != 0)
    {
      delay(timeTropfen2);

      //Ventil 1 und Ventil 2
      digitalWrite(Ventil1, HIGH);
      digitalWrite(Ventil2, HIGH); 
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      digitalWrite(Ventil2, LOW);
      //Serial.println("Ventil 1 und Ventil 2 kurz geoeffnet"); //Später auskommentieren

      //Blitz 2
      if(blitztime2 != 0)
      {
        delay(blitztime2);
        digitalWrite(Blitz2, HIGH);
        //Serial.println("Blitz 2 ausgeloest"); //Später auskommentieren 
      }
    }

    //Tropfen 3
    if(timeTropfen3 != 0)
    {
      delay(timeTropfen3);

      //Ventil 1 und Ventil 2
      digitalWrite(Ventil1, HIGH);
      digitalWrite(Ventil2, HIGH); 
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      digitalWrite(Ventil2, LOW);
      //Serial.println("Ventil 1 und Ventil 2 kurz geoeffnet"); //Später auskommentieren

      //Blitz 3
      if(blitztime3 != 0)
      {
        delay(blitztime3);
        digitalWrite(Blitz3, HIGH);
        //Serial.println("Blitz 3 ausgeloest"); //Später auskommentieren 
      }
    }    
    break;


    ////////Ventil 1/2/3
    case(2):
    //Kamera
    digitalWrite(wakeup, HIGH);
    delay(wakeuptime);
    digitalWrite(Kamera, HIGH);
    delay(kameraVerzoegerung);
    //Serial.println("Kamera Ausgeloest"); //Später auskommentieren

    //Ventil 1 und Ventil 2 und Ventil 3
    digitalWrite(Ventil1, HIGH);
    digitalWrite(Ventil2, HIGH); 
    digitalWrite(Ventil3, HIGH);
    delay(ventildelay);
    digitalWrite(Ventil1, LOW);
    digitalWrite(Ventil2, LOW);
    digitalWrite(Ventil3, LOW);
    //Serial.println("Ventil 1 und Ventil 2 und Ventil 3 kurz geoeffnet"); //Später auskommentieren

    //Blitz 1
    if(blitztime1 != 0)
    {
      delay(blitztime1);
      digitalWrite(Blitz1, HIGH);
      //Serial.println("Blitz 1 ausgeloest"); //Später auskommentieren 
    }   

    //Tropfen 2
    if(timeTropfen2 != 0)
    {
      delay(timeTropfen2);

      //Ventil 1 und Ventil 2 und Ventil 3
      digitalWrite(Ventil1, HIGH);
      digitalWrite(Ventil2, HIGH); 
      digitalWrite(Ventil3, HIGH);
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      digitalWrite(Ventil2, LOW);
      digitalWrite(Ventil3, LOW);
      //Serial.println("Ventil 1 und Ventil 2 und Ventil 3 kurz geoeffnet"); //Später auskommentieren

      //Blitz 2
      if(blitztime2 != 0)
      {
        delay(blitztime2);
        digitalWrite(Blitz2, HIGH);
        //Serial.println("Blitz 2 ausgeloest"); //Später auskommentieren 
      }
    }

    //Tropfen 3
    if(timeTropfen3 != 0)
    {
      delay(timeTropfen3);

      //Ventil 1 und Ventil 2 und Ventil 3
      digitalWrite(Ventil1, HIGH);
      digitalWrite(Ventil2, HIGH); 
      digitalWrite(Ventil3, HIGH);
      delay(ventildelay);
      digitalWrite(Ventil1, LOW);
      digitalWrite(Ventil2, LOW);
      digitalWrite(Ventil3, LOW);
      //Serial.println("Ventil 1 und Ventil 2 und Ventil 3 kurz geoeffnet"); //Später auskommentieren

      //Blitz 3
      if(blitztime3 != 0)
      {
        delay(blitztime3);
        digitalWrite(Blitz3, HIGH);
        //Serial.println("Blitz 3 ausgeloest"); //Später auskommentieren 
      }
    }    
    break;
  }

  delay(canonBlitzVerzoegerung);
  
  digitalWrite(wakeup, LOW);
  digitalWrite(Kamera, LOW);
  digitalWrite(Blitz1, LOW); 
  digitalWrite(Blitz2, LOW);
  digitalWrite(Blitz3, LOW);

  Serial.println("Bild wurde aufgenommen");

  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("Bild aufgenommen");

  delay(2000);
  lcd.clear();
}


//Momentaufnahme
void momentaufnahme()
{
  bool warte = true;
  
  Serial.println("\n");
  Serial.println("Umgebung wird kalibriert");
  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Umgebung wird");

  lcd.setCursor(0, 1);
  lcd.print("kalibriert");
  
  //Auf Umgebung Kalibrieren
  sensorWert = 0;

  for(int wert = 0; wert < anzahlMessungen; wert++)
  {
    wertRegister[wert] = analogRead(Sensor);
    delay(10);
  }

  for(int wert = 0; wert < anzahlMessungen; wert++)
  {
    sensorWert = wertRegister[wert] + sensorWert;
  }

  sensorWert = sensorWert / anzahlMessungen;

  minSensorWert = sensorWert - sensorempfindlichkeit;
  maxSensorWert = sensorWert + sensorempfindlichkeit;

  Serial.println("Anlage bereit fuer Bild");
  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Kalibrierung");

  lcd.setCursor(0, 1);
  lcd.print("abgeschlossen");
  delay(1000);
  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Kamera wird");

  lcd.setCursor(0, 1);
  lcd.print("ausgeloest in:");

  lcd.setCursor(0, 3);
  lcd.print("3");
  delay(1000);

  lcd.setCursor(0, 3);
  lcd.print("2");
  delay(1000);

  lcd.setCursor(0, 3);
  lcd.print("1");
   
  //Kamera und evtl Blitz aus Schlafmodus holen
  digitalWrite(wakeup, HIGH);
  delay(wakeuptime);
  
  //Kamera auslösen
  digitalWrite(Kamera, HIGH);
  Serial.println("Kamera wird ausgeloest");
  delay(kameraVerzoegerung);
  
  lcd.setCursor(0, 3);
  lcd.print("-------0--------");
   
  //Blitz auslösen
  do
  {
    sensorWert = analogRead(Sensor);
    
    if((sensorWert < minSensorWert)or(sensorWert > maxSensorWert))
    {
      delay(blitztime1);
      digitalWrite(Blitz1, HIGH);
      digitalWrite(Blitz2, HIGH);
      digitalWrite(Blitz3, HIGH);
      delay(40);
      //Serial.println("Blitz wurde ausgeloest"); //Später auskommentieren
      warte = false;     
    }
    
  }while(warte);

  digitalWrite(Kamera, LOW);
  digitalWrite(wakeup, LOW);
  digitalWrite(Blitz1, LOW);
  digitalWrite(Blitz2, LOW);
  digitalWrite(Blitz3, LOW);
  
  Serial.println("Bild wurde aufgenommen");
  Serial.println("\n");

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Bild aufgenommen");

  delay(2000);
  lcd.clear();
}


//Tropfenexplosion
void tropfenexplosion()
{
  Serial.println("\n");
  Serial.println("Bild wird aufgenommen");
  
  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("Bild wird");

  lcd.setCursor(0, 1);
  lcd.print("aufgenommen");

  lcd.setCursor(0, 3);
  lcd.print("Prozess laeuft..");


  //Kamera
  digitalWrite(wakeup, HIGH);
  delay(wakeuptime);
  digitalWrite(Kamera, HIGH);
  delay(kameraVerzoegerung); //Delay damit Kamera genug Zeit hat auf zu nehmen
  //Serial.println("Kamera Ausgeloest"); //Später auskommentieren

  //Ventil 1 (Lautsprecher)
  digitalWrite(Ventil1, HIGH);
  //Serial.println("Impuls auf Lautsprecher abgegeben"); //Später auskommentieren

  delay(blitztime1);
  digitalWrite(Blitz1, HIGH);
  delay(40); //Delay damit Blitz und Lautsprecher genug Zeit haben zu schalten
  //Serial.println("Blitz 1 ausgeloest"); //Später auskommentieren 

  digitalWrite(Ventil1, LOW);
  digitalWrite(wakeup, LOW);
  digitalWrite(Kamera, LOW);
  digitalWrite(Blitz1, LOW); 


  Serial.println("Bild wurde aufgenommen");

  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("Bild aufgenommen");

  delay(2000);
  lcd.clear();
}


//Encoder
int encoder(int)
{
  newPosition = rotary.rotaryDelta();
  
  if (newPosition != oldPosition) 
  {
    encoderveraenderung =  newPosition - oldPosition;
    
    oldPosition = newPosition;
    //Serial.println("Encoderveraenderung");   
  }

  return encoderveraenderung;
}

 
//LCD aktuallisieren 1 (Tropfeneinstellung)
void LCDaktuell1()
{
 if ((unsigned long)(millis() - LCDpreviousMillis) >= LCDinterval) //LCD aktualisieren alle paar (LCDinterval)ms
 {
  LCDpreviousMillis = millis();

  //Wechsle zwischen den verschiedenen Delay-Einstellugen
  if(rotary.buttonClicked() == true) //90ms delay. Für korrektes Taster einlesen (Debounce)
  {
    //Serial.println("Encoderknopf betaetigt");
    
    if(wechselpos >= 11)
    {
      wechselpos = 0;
    }
    else
    {
      wechselpos = wechselpos + 1;
    }
  }

  switch(wechselpos) //Welches Delay verändert/angezeigt werden sollte
  {
    case(0): //Blitz1time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Blitz 1:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);

      blitztime1 = blitztime1 + encoderveraenderung * stepsize;
      
      if(blitztime1 < 0)
      {
        blitztime1 = 0;
      }
      lcd.print(blitztime1);
      
      encoderveraenderung = 0;
    break;


    case(1): //Blitz2time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Blitz 2:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);
      
      blitztime2 = blitztime2 + encoderveraenderung * stepsize;
      
      if(blitztime2 < 0)
      {
        blitztime2 = 0;
      }
      lcd.print(blitztime2);
      
      encoderveraenderung = 0;
    break;


    case(2): //Blitz3time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Blitz 3:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);
      
      blitztime3 = blitztime3 + encoderveraenderung * stepsize;
      
      if(blitztime3 < 0)
      {
        blitztime3 = 0;
      }
      lcd.print(blitztime3);
      
      encoderveraenderung = 0;
    break;


    case(3): //Tropfen2time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Tropfen 2:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);
      
      timeTropfen2 = timeTropfen2 + encoderveraenderung * stepsize;

      if(timeTropfen2 < 0)
      {
        timeTropfen2 = 0;
      }
      lcd.print(timeTropfen2);
      
      encoderveraenderung = 0;
    break;


    case(4): //Tropfen3time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Tropfen 3:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);
      
      timeTropfen3 = timeTropfen3 + encoderveraenderung * stepsize;
      
      if(timeTropfen3 < 0)
      {
        timeTropfen3 = 0;
      }
      lcd.print(timeTropfen3);
      
      encoderveraenderung = 0;   
    break; 


    case(5): //Anzahl Ventile
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Anzahl Ventile:");

      lcd.setCursor(0, 1);
      lcd.print((anzahlVentile + 1));
      
      anzahlVentile = anzahlVentile + encoderveraenderung * (stepsize / stepsize);

      if(anzahlVentile < 0)
      {
        anzahlVentile = 0;
      }
      
      if(anzahlVentile > 2)
      {
        anzahlVentile = 2;
      }
      
      encoderveraenderung = 0;   
    break;

    case(6): //Ventil 1 öffnen
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Ventil 1");
      lcd.setCursor(0, 1);
      lcd.print("oeffnen:");
      
      ventil1Test = ventil1Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil1Test < 0)
      {
        ventil1Test = 0;
      }
      
      if(ventil1Test > 2)
      {
        digitalWrite(Ventil1, HIGH);
        
        lcd.setCursor(0, 3);
        lcd.print("Ventil 1 offen");
        
        if(ventil1Test > 4)
        {
          ventil1Test = 4;
        }
      }
      else
      {
        digitalWrite(Ventil1, LOW);
      }
      
      encoderveraenderung = 0;   
    break; 


    case(7): //Ventil 2 öffnen
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Ventil 2");
      lcd.setCursor(0, 1);
      lcd.print("oeffnen:");
      
      ventil2Test = ventil2Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil2Test < 0)
      {
        ventil2Test = 0;
      }
            
      if(ventil2Test > 2)
      {
        digitalWrite(Ventil2, HIGH);
        
        lcd.setCursor(0, 3);
        lcd.print("Ventil 2 offen");
        
        if(ventil2Test > 4)
        {
          ventil2Test = 4;
        }
      }
      else
      {
        digitalWrite(Ventil2, LOW);
      }
      
      encoderveraenderung = 0;   
    break; 


    case(8): //Ventil 3 öffnen
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Ventil 3");
      lcd.setCursor(0, 1);
      lcd.print("oeffnen:");
      
      ventil3Test = ventil3Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil3Test < 0)
      {
        ventil3Test = 0;
      }
            
      if(ventil3Test > 2)
      {
        digitalWrite(Ventil3, HIGH);
        
        lcd.setCursor(0, 3);
        lcd.print("Ventil 3 offen");
        
        if(ventil3Test > 4)
        {
          ventil3Test = 4;
        }
      }
      else
      {
        digitalWrite(Ventil3, LOW);
      }
      
      encoderveraenderung = 0;   
    break;


    case(9): //Testtropfen Ventil 1
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Testtropfen");
      lcd.setCursor(0, 1);
      lcd.print("Ventil 1:");      

      ventil1Test = ventil1Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil1Test < 0)
      {
        ventil1Test = 0;
      }
            
      if(ventil1Test > 2)
      {
        digitalWrite(Ventil1, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil1, LOW);
         
        lcd.setCursor(0, 3);
        lcd.print("Test ausgefuehrt"); 
        
        Serial.println("Testtropfen aus Ventil 1 ausgeführt");   
           
        delay(1000);
        
        if(ventil1Test > 4)
        {
          ventil1Test = 4;
        }
      }
    
      encoderveraenderung = 0;
    break;


    case(10): //Testtropfen Ventil 2
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Testtropfen");
      lcd.setCursor(0, 1);
      lcd.print("Ventil 2:");      

      ventil2Test = ventil2Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil2Test < 0)
      {
        ventil2Test = 0;
      }
            
      if(ventil2Test > 2)
      {
        digitalWrite(Ventil2, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil2, LOW);
         
        lcd.setCursor(0, 3);
        lcd.print("Test ausgefuehrt"); 
        
        Serial.println("Testtropfen aus Ventil 2 ausgeführt");   
           
        delay(1000);
        
        if(ventil2Test > 4)
        {
          ventil2Test = 4;
        }
      }
    
      encoderveraenderung = 0;
    break;


    case(11): //Testtropfen Ventil 3
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Testtropfen");
      lcd.setCursor(0, 1);
      lcd.print("Ventil 3:");      

      ventil3Test = ventil3Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil3Test < 0)
      {
        ventil3Test = 0;
      }
            
      if(ventil3Test > 2)
      {
        digitalWrite(Ventil3, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil3, LOW);
         
        lcd.setCursor(0, 3);
        lcd.print("Test ausgefuehrt"); 
        
        Serial.println("Testtropfen aus Ventil 3 ausgeführt");   
           
        delay(1000);
        
        if(ventil3Test > 4)
        {
          ventil3Test = 4;
        }
      }
    
      encoderveraenderung = 0;
    break;
  }
 }
}


//LCD aktuallisieren 2 (Momentaufnahmeneinstellung)
void LCDaktuell2()
{
 if ((unsigned long)(millis() - LCDpreviousMillis) >= LCDinterval) //LCD aktualisieren alle paar (LCDinterval)ms
 {
  LCDpreviousMillis = millis();

  //Wechsle zwischen den verschiedenen Einstellugen
  if(rotary.buttonClicked() == true) //90ms delay. Für korrektes Taster einlesen (Debounce)
  {
    //Serial.println("Encoderknopf betaetigt");
    
    if(wechselpos >= 1)
    {
      wechselpos = 0;
    }
    else
    {
      wechselpos = wechselpos + 1;
    }
  }

  switch(wechselpos) //Welches Delay verändert/angezeigt werden sollte
  {
    case(0): //Blitzdelay
    lcd.clear();
      
    lcd.setCursor(0, 0);
    lcd.print("Delay Blitz:");

    lcd.setCursor(14, 1);
    lcd.print("ms");

    lcd.setCursor(0, 1);

    blitztime1 = blitztime1 + encoderveraenderung * stepsize;
      
    if(blitztime1 < 0)
    {
      blitztime1 = 0;
    }
    lcd.print(blitztime1);
      
    encoderveraenderung = 0;
    break;

    case(1): //Sensorempfindlichkeit
    lcd.clear();
      
    lcd.setCursor(0, 0);
    lcd.print("Sensor");

    lcd.setCursor(0, 1);
    lcd.print("Empfindlichkeit:");

    lcd.setCursor(0, 3);

    sensorempfindlichkeit = sensorempfindlichkeit + encoderveraenderung;
      
    if(sensorempfindlichkeit < 0)
    {
      sensorempfindlichkeit = 0;
    }
    lcd.print(sensorempfindlichkeit);
      
    encoderveraenderung = 0;
    break;
  }
 }  
}


//LCD aktuallisieren 3 (Tropfenexplosion)
void LCDaktuell3()
{
 if ((unsigned long)(millis() - LCDpreviousMillis) >= LCDinterval) //LCD aktualisieren alle paar (LCDinterval)ms
 {
  LCDpreviousMillis = millis();

  //Wechsle zwischen den verschiedenen Delay-Einstellugen
  if(rotary.buttonClicked() == true) //90ms delay. Für korrektes Taster einlesen (Debounce)
  {
    //Serial.println("Encoderknopf betaetigt");
    
    if(wechselpos >= 1)
    {
      wechselpos = 0;
    }
    else
    {
      wechselpos = wechselpos + 1;
    }
  }

  switch(wechselpos) //Welches Delay verändert/angezeigt werden sollte
  {
    case(0): //Blitz1time
      lcd.clear();
      
      lcd.setCursor(0, 0);
      lcd.print("Delay Blitz 1:");

      lcd.setCursor(14, 1);
      lcd.print("ms");

      lcd.setCursor(0, 1);

      blitztime1 = blitztime1 + encoderveraenderung * stepsize;
      
      if(blitztime1 < 0)
      {
        blitztime1 = 0;
      }
      lcd.print(blitztime1);
      
      encoderveraenderung = 0;
    break;


    case(1): //Lautsprecher testen
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Lautsprecher");
      lcd.setCursor(0, 1);
      lcd.print("testen:");      

      ventil1Test = ventil1Test + encoderveraenderung * (stepsize / stepsize);

      if(ventil1Test < 0)
      {
        ventil1Test = 0;
      }
            
      if(ventil1Test > 2)
      {
        digitalWrite(Ventil1, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil1, LOW);
         
        lcd.setCursor(0, 3);
        lcd.print("Test ausgefuehrt"); 
        
        Serial.println("Impuls auf Lautsprecher abgegeben");   
           
        delay(1000);
        
        if(ventil1Test > 4)
        {
          ventil1Test = 4;
        }
      }
    
      encoderveraenderung = 0;
    break;
  }
 }
}


//System testen
void systemTesten(void)
{
    if(rotary.buttonClicked() == true)
    {
      testNr = testNr + 1;
      //Serial.println("Testwechsel");  
      if(testNr > 10)
      {
        testNr = 0;
      } 
    }

  delay(20);
  
  switch(testNr)
  {
    //Kameratest
    case(0):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Kamera testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(wakeup, HIGH);
        digitalWrite(Kamera, HIGH);
        delay(kameraVerzoegerung);

        lcd.setCursor(0, 2);
        lcd.print("Kamera");
        lcd.setCursor(0, 3);
        lcd.print("ausgeloest");
        Serial.println("Kamera wurde ausgeloest");
        delay(2000);
      }
      
      digitalWrite(wakeup, LOW);
      digitalWrite(Kamera, LOW);
    break;

    //Blitz 1 Test
    case(1):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Blitz 1 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(wakeup, HIGH);
        digitalWrite(Blitz1, HIGH);
        delay(40);

        lcd.setCursor(0, 2);
        lcd.print("Blitz 1");
        lcd.setCursor(0, 3);
        lcd.print("ausgeloest");
        Serial.println("Blitz 1 wurde ausgeloest");
        delay(2000);
      }
      
      digitalWrite(wakeup, LOW);
      digitalWrite(Blitz1, LOW);
    break;

    //Blitz 2 Test
    case(2):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Blitz 2 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(wakeup, HIGH);
        digitalWrite(Blitz2, HIGH);
        delay(40);

        lcd.setCursor(0, 2);
        lcd.print("Blitz 2");
        lcd.setCursor(0, 3);
        lcd.print("ausgeloest");
        Serial.println("Blitz 2 wurde ausgeloest");
        delay(2000);
      }
      
      digitalWrite(wakeup, LOW);
      digitalWrite(Blitz2, LOW);
    break;

    //Blitz 3 Test
    case(3):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Blitz 3 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(wakeup, HIGH);
        digitalWrite(Blitz3, HIGH);
        delay(40);

        lcd.setCursor(0, 2);
        lcd.print("Blitz 3");
        lcd.setCursor(0, 3);
        lcd.print("ausgeloest");
        Serial.println("Blitz 3 wurde ausgeloest");
        delay(2000);
      }
      
      digitalWrite(wakeup, LOW);
      digitalWrite(Blitz3, LOW);
    break;

    //Ventil 1 Test
    case(4):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ventil 1 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {       
        digitalWrite(Ventil1, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil1, LOW); 

        lcd.setCursor(0, 2);
        lcd.print("Ventil 1");
        lcd.setCursor(0, 3);
        lcd.print("eingeschaltet");
        Serial.println("Ventil 1 wurde eingeschaltet");
        delay(1000);               
      }
    break;

    //Ventil 2 Test
    case(5):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ventil 2 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(Ventil2, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil2, LOW); 

        lcd.setCursor(0, 2);
        lcd.print("Ventil 2");
        lcd.setCursor(0, 3);
        lcd.print("eingeschaltet");
        Serial.println("Ventil 2 wurde eingeschaltet");
        delay(1000);
      }
    break;

    //Ventil 3 Test
    case(6):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ventil 3 testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(Ventil3, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil3, LOW); 

        lcd.setCursor(0, 2);
        lcd.print("Ventil 3");
        lcd.setCursor(0, 3);
        lcd.print("eingeschaltet");
        Serial.println("Ventil 3 wurde eingeschaltet");
        delay(1000);
      }
    break;

    //Aus Schlafmodus holen testen
    case(7):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Antischlafmodus");
      lcd.setCursor(0, 1);
      lcd.print("testen:");

      if(digitalRead(taster_ausloesen) == 0)
      {
        digitalWrite(wakeup, HIGH);
        delay(1000);

        lcd.setCursor(0, 2);
        lcd.print("Kamera & Blitz");
        lcd.setCursor(0, 3);
        lcd.print("wurden geweckt");
        Serial.println("Kamera & Blitz wurden geweckt");
        delay(2000);
      }
      
      digitalWrite(wakeup, LOW);
    break;

    //Lichtschranke
    case(8):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Lichtschranke");
      lcd.setCursor(0, 1);
      lcd.print("Tropfen testen:");


      if(digitalRead(L1) == 1)
      {
        lcd.setCursor(0, 3);
        lcd.print("Weg ist zu"); 
        Serial.println("Lichtschranke ist zu");       
      }
      else
      {
        lcd.setCursor(0, 3);
        lcd.print("Weg ist frei");
        Serial.println("Lichtschranke ist frei");        
      }      
    break;  

    //Sensortest
    case(9):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Sensor");
      lcd.setCursor(0, 1);
      lcd.print("testen:");

      lcd.setCursor(0, 3);
      lcd.print(analogRead(Sensor)); 

      Serial.println("SensorWert: ");
      Serial.println(analogRead(Sensor));
    break; 

    //Lautsprecher Test
    case(10):
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Lautsprecher");

      lcd.setCursor(0, 1);
      lcd.print("testen:");      

      if(digitalRead(taster_ausloesen) == 0)
      {       
        digitalWrite(Ventil1, HIGH);
        delay(ventildelay);
        digitalWrite(Ventil1, LOW); 

        lcd.setCursor(0, 2);
        lcd.print("Impuls");
        lcd.setCursor(0, 3);
        lcd.print("abgegeben");
        Serial.println("Impuls auf Lautsprecher wurde abgegeben");
        delay(1000);               
      }
    break;          
  }
  
}


//Initialisierung
void initialisiere(void)
{
  lcd.setCursor(0, 0);
  lcd.print("Kamera/Blitz");
    
  lcd.setCursor(0, 1);
  lcd.print("Steuereinheit");

  lcd.setCursor(0, 2);
  lcd.print("----------------");

  lcd.setCursor(0, 3);
  lcd.print("Einsatzbereit");

  delay(3000);
  delay(3000); //Keine Ahnung warum der Arduino beim Aufstarten das erste Delay ignoriert. Dieses Delay hier ist, damit der Arduino es sicher macht.

  Serial.println("Kamera Blitz Steuereinheit gestartet:");

  waehlzeit = waehldelay + millis();

  //Schlaufe
  do
  {

    if(rotary.buttonClicked() == true)
    {
      prg = prg + 1; //Programm wechseln
      Serial.println("Programm wechseln");  
      if(prg > 3)
      {
        prg = 0;
      } 
    }
   
    if(prg == 0)
    {
      lcd.setCursor(0, 0);
      lcd.print("->System testen");
   
      lcd.setCursor(0, 1);
      lcd.print("  Tropfen Bilder");

      lcd.setCursor(0, 2);
      lcd.print("  Momentaufnahme");

      lcd.setCursor(0, 3);
      lcd.print("  Tropfenexpl."); 
    }

    if(prg == 1)
    {
      lcd.setCursor(0, 0);
      lcd.print("  System testen");
   
      lcd.setCursor(0, 1);
      lcd.print("->Tropfen Bilder");

      lcd.setCursor(0, 2);
      lcd.print("  Momentaufnahme");

      lcd.setCursor(0, 3);
      lcd.print("  Tropfenexpl.");   
    }    

    if(prg == 2)
    {
      lcd.setCursor(0, 0);
      lcd.print("  System testen");
   
      lcd.setCursor(0, 1);
      lcd.print("  Tropfen Bilder");

      lcd.setCursor(0, 2);
      lcd.print("->Momentaufnahme");

      lcd.setCursor(0, 3);
      lcd.print("  Tropfenexpl.");  
    }

    if(prg == 3)
    {
      lcd.setCursor(0, 0);
      lcd.print("  System testen");
   
      lcd.setCursor(0, 1);
      lcd.print("  Tropfen Bilder");

      lcd.setCursor(0, 2);
      lcd.print("  Momentaufnahme");

      lcd.setCursor(0, 3);
      lcd.print("->Tropfenexpl.");      
    }
    
  }while(waehlzeit > millis());

  if(prg == 0)
  {

    Serial.println("Programm _System testen_ wurde gewaehlt");  
  }

  if(prg == 1)
  {

    Serial.println("Programm _Tropfen Bilder_ wurde gewaehlt");  
  }    

  if(prg == 2)
  {

    Serial.println("Programm _Momentaufnahme_ wurde gewaehlt");  
  }

  if(prg == 3)
  {

    Serial.println("Programm _Tropfenexplosion_ wurde gewaehlt");  
  }  

  delay(2000);
  lcd.clear();  
}
